@@ -27,10 +27,7 @@ class IfRelationshipNode(template.Node):
             return self.nodelist_false.render(context)
 
         try:
-            status = RelationshipStatus.objects.get(
-                Q(from_slug=self.status) | 
-                Q(to_slug=self.status) | 
-                Q(symmetrical_slug=self.status))
+            status = RelationshipStatus.objects.by_slug(self.status)
         except RelationshipStatus.DoesNotExist:
             raise template.TemplateSyntaxError('RelationshipStatus not found')
         
@@ -39,10 +36,11 @@ class IfRelationshipNode(template.Node):
         elif status.to_slug == self.status:
             val = to_user.relationships.exists(from_user, status)
         else:
-            val = from_user.relationships.symmetrical_exists(to_user, status)
+            val = from_user.relationships.exists(to_user, status, symmetrical=True)
             
         if val:
             return self.nodelist_true.render(context)
+
         return self.nodelist_false.render(context)
 
 @register.tag
@@ -66,7 +64,7 @@ def if_relationship(parser, token):
     """
     bits = list(token.split_contents())
     if len(bits) != 4:
-        raise TemplateSyntaxError, "%r takes 3 arguments:\n" % \
+        raise TemplateSyntaxError, "%r takes 3 arguments:\n%s" % \
             (bits[0], if_relationship.__doc__)
     end_tag = 'end' + bits[0]
     nodelist_true = parser.parse(('else', end_tag))
@@ -84,10 +82,11 @@ def add_relationship_url(user, status):
     """
     Generate a url for adding a relationship on a given user.  ``user`` is a
     User object, and ``status`` is either a relationship_status object or a 
-    string matching the ``from_slug`` of a RelationshipStatus
+    string denoting a RelationshipStatus
     
-    Usage:
-    href="{{ user|add_relationship_url:"following" }}"
+    Usage::
+    
+        href="{{ user|add_relationship_url:"following" }}"
     """
     if isinstance(status, RelationshipStatus):
         status = status.from_slug
@@ -99,10 +98,11 @@ def remove_relationship_url(user, status):
     """
     Generate a url for removing a relationship on a given user.  ``user`` is a
     User object, and ``status`` is either a relationship_status object or a 
-    string matching the ``from_slug`` of a RelationshipStatus
+    string denoting a RelationshipStatus
     
-    Usage:
-    href="{{ user|remove_relationship_url:"following" }}"
+    Usage::
+    
+        href="{{ user|remove_relationship_url:"following" }}"
     """
     if isinstance(status, RelationshipStatus):
         status = status.from_slug
